/* This program is arcs2ps.c (``arcs to postscript'') -- it requires the 
   output file (probably called ``arcinfo.dat'') of the program statprot, 
   statprotC, statdna, or statdnaC.  The program is intended to take the 
   information about arc probabilities from the output file and produce a 
   pretty postscript picture.  This program requires a file ``control.dat'' 
   for black and white output or a file ``controlcol.dat'' for color output.
   The progam can be compiled by typing:

   cc arcs2ps.c -lm -o arcs2bwps (for black and white postscript file)
   cc arcs2ps.c -DCOLOR -lm -o arcs2cps (for color postscript file)

   More details can be found in the readme file.
*/

#include <math.h>
#include <stdio.h>

#define min2(A,B) ((A) < (B) ? (A) : (B))
#define min3(A,B,C) ((A) < (B) ? min2(A,C) : min2(B,C)) 
#define MAXARCTYPES 10 /* Allow for this many types of arcs */
#define DEFTHICK 2.0 /* Thickness of all lines when drawflag=0 */
#define MAXLEN 5000 /* maximum length of a sequence */

int lowarc,higharc; /* index of lowest and highest arc types */
int hashflag ;  /* 1 means draw only hash marks, 0 means draw sequence */
int hashinc;  /* how many bases per hash mark if hashflag== 1 */
int drawflag;  /* 1 means draw line thickness in proportion to probability,
                      0 means draw all lines above a threshold probability
                        with the same thickness */
float threshold; /* When drawflag=1, what's the thinnest linewidth
                         that should be drawn?  When drawflag=0, what's
                         the smallest probability that should be drawn? */
float mwidcon; /* maximum width of a line 
                        compared to distance between nodes */
float hormar; /* number of inches per horizontal margin */
float vermar;  /* number of inches per vertical margin */
int decx[MAXARCTYPES],decy[MAXARCTYPES];
#ifdef COLOR
float arcred[MAXARCTYPES], arcgreen[MAXARCTYPES],arcblue[MAXARCTYPES];
#else
float arcgray[MAXARCTYPES];
#endif
char arcinfo[30], outps[30];
FILE *infile, *outfile;

int xlen,ylen; /* lengths of the sequences to be drawn on the x and y axes */
float nxbase,nybase;
float inc,maxwidth;  
char seq1[MAXLEN],seq2[MAXLEN]; /* the two sequences */

main()
{
   void rdcontrol();
   void rdarcinfo();
   void drawarc();
   void describearc();

   float fontsize;
   int oldx,oldy,newd,newx,newy,track1,track2;
   float linwid,oldprob,newprob;
   float xbase,ybase,yspace,xspace;
   float thisprob[MAXARCTYPES], dprob[4];

#ifdef COLOR
   float red, green, blue;
#else
   float gray;
#endif

   rdcontrol();
   xbase=(hormar*72.0);
   ybase=((11.0-vermar)*72.0);
   yspace=((11.0-2.0*vermar)*72.0);
   xspace=((8.5-2.0*hormar)*72.0);

   rdarcinfo();
   outfile= fopen(outps,"w");
   fprintf(outfile,"%%!PS-Adobe-2.0\n\n");
   fprintf(outfile,"/Courier-Bold findfont\n");

   /* do the borders of the postscript output file */
   if (hashflag==0)
   {
      /* draw in the sequences */
      inc= min2(xspace/(xlen+1.0), yspace/(ylen+1.0));
      maxwidth=inc*mwidcon;
      fontsize= inc;
      nxbase= xbase+inc;
      nybase= ybase-inc;
      fprintf(outfile,"%4.2f scalefont setfont\n",fontsize);
      for(track1=1;track1<=xlen;track1++)
      {
         fprintf(outfile,
            "%5.2f %5.2f moveto\n", xbase-fontsize/4.0+inc*(0.5+track1),
            ybase-0.67*fontsize); 
         fprintf(outfile,"(%1c) show\n\n", seq1[track1]);
      }
      for(track2=1;track2<=ylen;track2++)
      {
         fprintf(outfile,"%5.2f %5.2f moveto\n", xbase+inc/2.0 -fontsize/4.0,
            ybase-fontsize/3.0-inc*(0.5+track2)); 
         fprintf(outfile, "(%1c) show\n\n", seq2[track2]);
      }
   }
   else /* hashflag==1 */
   {
      /* draw only hash marks */
      fontsize=12.0;
      inc= min2((xspace-2*fontsize)/(xlen+1.0),(yspace-2*fontsize)/(ylen+1.0));
      maxwidth=inc*mwidcon;
      fprintf(outfile, "%4.2f scalefont setfont\n",fontsize);
      nxbase=xbase+2.0*fontsize;
      nybase=ybase-2.0*fontsize;
      track1=0;
      while (track1<=xlen)
      {
         fprintf(outfile, "newpath\n");
         fprintf(outfile, "%5.2f %5.2f moveto\n", 
            xbase+inc*track1+2.0*fontsize, ybase-1.2*fontsize);
         fprintf(outfile, "0.0 %5.2f rlineto\n", -0.6*fontsize); 
         fprintf(outfile, "1 setlinewidth\n");
         fprintf(outfile, "stroke\n");
      
         fprintf(outfile, "%5.2f %5.2f moveto\n", 
            xbase+inc*track1+2.0*fontsize, ybase -fontsize);
         fprintf(outfile,"(%d) stringwidth pop -2.0 div 0.0 rmoveto\n", track1);
         fprintf(outfile,"(%d) show\n\n",track1);

         track1 += hashinc;
      }
      track2=0;
      while (track2 <= ylen)
      {
         fprintf(outfile,"newpath\n");
         fprintf(outfile,"%5.2f %5.2f moveto\n", xbase+1.2*fontsize, 
            ybase-inc*track2-2.0*fontsize);
         fprintf(outfile,"%5.2f 0.0 rlineto\n", 0.6*fontsize);
         fprintf(outfile,"1 setlinewidth\n");
         fprintf(outfile,"stroke\n\n");

         fprintf(outfile,"%5.2f %5.2f moveto\n", 
            xbase+fontsize, ybase-inc*track2 -2.3*fontsize);
         fprintf(outfile,"(%d) stringwidth pop -1.0 div 0.0 rmoveto\n", track2);
         fprintf(outfile,"(%d) show\n\n",track2);
      
         track2+= hashinc;
      }
   }
   /* The borders are finished.  Start drawing the arcs */
   newprob=0.0;
   oldprob=0.0;
   oldx=0;
   oldy=0;
   newx=oldx;
   newy=oldy;
   newd=lowarc;
   for(track1=0;track1<=higharc;track1++)
   {
      thisprob[track1]=0.0;
   }
   for(track1=0;track1<=3;track1++)
   {
      dprob[track1]=0.0;
   }

   while (newprob >=0.0)
   {
      while(newx==oldx && newy==oldy)
      {
         thisprob[newd]=newprob;
         track1 = 2*decx[newd]+decy[newd];
         if(track1 > 0)
         {
            dprob[track1] += newprob;
         }
         fscanf(infile, "%d%d%d%f", &newx,&newy,&newd,&newprob);
      }
      for(track1=1; track1 <=3; track1++)
      {
         if(dprob[track1] > 0.0)
         {
#ifdef COLOR
            describearc(thisprob, track1, dprob[track1], &linwid, &red,
                        &green, &blue); 
#else
            describearc(thisprob, track1, dprob[track1], &linwid, &gray); 
#endif
            if(linwid>0.0)
            {
#ifdef COLOR
               fprintf(outfile,"%4.2f %4.2f %4.2f setrgbcolor\n", red,
                       green, blue);
#else
               fprintf(outfile,"%4.2f setgray\n", gray);
#endif
               drawarc(oldx,oldy,track1,linwid);
            }
         }
         dprob[track1]=0.0;
      }
      for(track1=lowarc; track1 <= higharc; track1++)
      {
         thisprob[track1]=0.0;
      }   
      oldx=newx;
      oldy=newy;
   }
   fprintf(outfile,"showpage\n");
   fclose(outfile);
   fclose(infile);
}

/* Reads information in file ``control.dat'' or ``controlcol.dat'' */
void 
rdcontrol()
{
   int dummy,ktrack,ktrack2;
   FILE *control;
   
#ifdef COLOR
   control=fopen("controlcol.dat","r");
#else
   control=fopen("control.dat","r");
#endif
   fscanf(control,"%d", &lowarc);
   while (getc(control) != '\n');
   fscanf(control,"%d", &higharc);
   while (getc(control) != '\n');
   while (getc(control) != '\n');
   for(ktrack=lowarc;ktrack<=higharc;ktrack++)
   {
      fscanf(control,"%d",&dummy);
#ifdef COLOR
       fscanf(control,"%d%d%f%f%f", &decx[dummy], &decy[dummy], &arcred[dummy],
              &arcgreen[dummy], &arcblue[dummy]);
#else
      fscanf(control,"%d%d%f", &decx[dummy], &decy[dummy], &arcgray[dummy]);
#endif
      while (getc(control) != '\n');
   } 
   fscanf(control, "%f", &threshold);
   while (getc(control) != '\n');
   fscanf(control, "%d", &drawflag);
   while (getc(control) != '\n');
   fscanf(control, "%f", &mwidcon);
   while (getc(control) != '\n');
   fscanf(control, "%f", &hormar);
   while (getc(control) != '\n');
   fscanf(control, "%f", &vermar);
   while (getc(control) != '\n');
   fscanf(control, "%d", &hashflag);
   while (getc(control) != '\n');
   fscanf(control, "%d", &hashinc);
   while (getc(control) != '\n');
   fscanf(control, "%s\n", arcinfo);
   while (getc(control) != '\n');
   fscanf(control, "%s\n", outps);
   while (getc(control) != '\n');
   fclose(control); 
}

/* read the sequence lengths and then the sequences */ 
void 
rdarcinfo()
{
   int ttrack,ltrack;
   char c;

   infile=fopen(arcinfo,"r");
   fscanf(infile,"%d", &xlen);
   fscanf(infile,"%d", &ylen);

   ttrack=0;
   while ( ttrack < xlen)
   {
      c=getc(infile);
      if ((c >='A' && c<='Z') || c=='?')
      {
         ttrack++;
         seq1[ttrack]=c;
      }
   }
     
   ttrack=0;
   while ( ttrack < ylen)
   {
      c=getc(infile);
      if ((c >='A' && c<='Z') || c=='?')
      {
         ttrack++;
         seq2[ttrack]=c;
      }
   }
   while (getc(infile) != '>');
   while (getc(infile) != '\n');     
}

/* determine width and grayness (or color) of arc */
#ifdef COLOR
void
describearc(theprob, thed, thedprob, thewid, thered, thegreen, theblue)
#else
void
describearc(theprob, thed, thedprob, thewid, thegray)
#endif

int thed;
float *theprob, *thewid, thedprob;
#ifdef COLOR
float *thered, *theblue, *thegreen;
#else
float  *thegray;
#endif
{
   int xinc, yinc, ktrack;
   

   if (drawflag==1)
   {
      *thewid = maxwidth*thedprob;
      if ( *thewid < threshold) *thewid = 0.0;
   }
   else  
   {
      /* drawflag==0 */
      if (thedprob > threshold)
         *thewid = DEFTHICK;
      else
         *thewid = 0.0;
   }

   if ( *thewid > 0.0)
   {
      xinc= thed/2;
      yinc= thed%2;

#ifdef COLOR
      *thered = 0.0;
      *theblue = 0.0;
      *thegreen = 0.0;
#else
      *thegray = 0.0;
#endif

      for(ktrack=lowarc;ktrack<=higharc;ktrack++)
      {
         if(xinc==decx[ktrack] && yinc==decy[ktrack])
         {
#ifdef COLOR
            *thered += (theprob[ktrack]/thedprob) * arcred[ktrack];
            *thegreen += (theprob[ktrack]/thedprob) * arcgreen[ktrack];
            *theblue += (theprob[ktrack]/thedprob) * arcblue[ktrack];
#else
            *thegray += (theprob[ktrack]/thedprob) * arcgray[ktrack];
#endif
         }
      }
   }
}
  
/* draw an arc */
void 
drawarc(xspot, yspot, arcdir, arcwid)
int xspot,yspot, arcdir;
float arcwid;
{
   fprintf(outfile,"newpath\n");
   fprintf(outfile,"%5.2f %5.2f moveto\n", 
      nxbase+xspot*inc, nybase-yspot*inc);
   if(arcdir==3)
   {
      fprintf(outfile,"%5.2f %5.2f rlineto\n", -1.0*inc,inc);
   }
   else
   {
      if(arcdir==2)
      {
         fprintf(outfile,"0.0 %5.2f rlineto\n", inc);
      }
      else
      {
         if(arcdir==1)
         {
            fprintf(outfile,"%5.2f 0.0 rlineto\n", -1.0*inc);
         }
         else
         {
            printf("Trouble in routine arcdraw()\n");
         }
      }  
   }
   fprintf(outfile,"%3.1f setlinewidth\n", arcwid);
   fprintf(outfile,"stroke\n\n");
}

