/* To compile this program, type:

   cc medians.c arrayutil.c -lm -o medians

   This program reads the formatted file "seqs.est" to obtain median
   estimates of parameters (or parameter ratios) from a set of
   pairwise comparisons.  The file "seqs.est" is produced by the
   program "statdna" or "statprot". The first line of "seqs.est" contains the
   number of sequences and the second line contains the number of
   parameters in the model.  Then, the parameter estimates and
   standard errors from each pairwise comparison follow.
  
   The results of this program are designed to specify the constraints
   needed when distance-based from phylogeny inference from
   non-aligned sequences is done via the method of Thorne and
   Kishino (MBE, 1992, 9:1148--1162).  The results should determine
   the entries of the file "protmask.dat" or "dnamask.dat".
*/


#include <stdio.h>

#define min2(A,B) ((A) < (B) ? (A) : (B))
#define min3(A,B,C) ((A) < (B) ? min2(A,C) : min2(B,C)) 

int nparams, nseqs; /* number of model parameters and number of sequences
                       in data set */
double **data; /* contains the parameter estimates from pairwise
                  comparisons */

main()
{
   int veclength,ktrack,treatpar;
   double *dumvec;
   double median;

   void sort(); /* sorts values of a vector (in-place) from low to
                   high */
   void prepare(); /* create the vector to be copied by extracting
                      relevant quantities from the matrix "data" */
   void readdata(); /* read the formatted file "seqs.est" */
   extern double **dmat(), *dvec();
   extern void freedmat(), freedvec();

   readdata();
   veclength= (((nseqs-1)*nseqs)/2) -1; /* number of pairwise
                                           comparisons minus 1 */
   dumvec= dvec(veclength);
   for(ktrack=0;ktrack< nparams;ktrack++)
   {
      /* For parameter 2 (deletion parameter) and parameter 7 (within
         group substitution parameter), we want the median of 
         parameter 2  (or 7) divided by parameter 1 (the general 
         substitution rate).  For other parameters, we want the 
         unadulterated median. */
      if(ktrack==2 || ktrack==7)
      {
         treatpar=-1;  
      }
      else
      {
         treatpar=0;
      }
      /* put relevant pairwise estimates into the vector dumvec */
      prepare(dumvec,ktrack,treatpar); 
      /* sort dumvec from low values to high values */
      sort(dumvec,veclength);
      printf("The median of parameter ");
      if(treatpar<0)
      {
         printf("%2d divided by %2d ", ktrack, abs(treatpar));
      }
      else
      {
         if(treatpar>0)
         {
            printf("%2d divided by %2d ", treatpar, ktrack);
         }
         else
         {
            printf("%2d ", ktrack); 
         }
      }
      if(veclength%2 == 0)
      {
         median = dumvec[veclength/2];
      }
      else
      {
         median= (dumvec[(veclength+1)/2] +dumvec[(veclength-1)/2])/2.0;
      }
      printf("is %12.8E\n", median);

   }


   freedvec(dumvec, veclength);
   freedmat(data,nparams-1, veclength);
}

/* the following routines reads the pairwise comparison data from
   the file "seqs.est" */
void readdata()
{
   extern double **dmat();

   FILE *seqsest;
   int veclength, dum1, dum2, ktrack1, ktrack2;
   double dubdum, dum3;

   seqsest=fopen("seqs.est", "r");
   fscanf(seqsest,"%d%d", &nseqs, &nparams); 
   printf("Number of Sequences = %3d\n", nseqs);
   printf("Number of parameters = %2d\n", nparams);
   veclength= (((nseqs-1)*nseqs)/2) -1;
   printf("Number of pairwise distances= %3d\n", veclength+1);


   data=dmat(nparams-1,veclength);
   for(ktrack1=0;ktrack1<=veclength;ktrack1++)
   {
      fscanf(seqsest, "\n");
      fscanf(seqsest, "%d%d%E\n", &dum1, &dum2 , &dubdum);
      for(ktrack2=0;ktrack2<nparams;ktrack2++)
      {
         fscanf(seqsest, "%d%lf%lf\n", &dum1,&data[ktrack2][ktrack1], &dum3);
      }
   }
   fclose(seqsest);
}
 
/* sorts a vector "tdumvec" from lowest to highest value.  The vector
   "tdumvec" has a low index of 0 and a high index of "veclength" */
void sort(tdumvec,veclength)
int veclength;
double *tdumvec;
{
   int ktrack1,ktrack2;
   double hold;

   for(ktrack1=veclength;ktrack1>0 ;ktrack1--)
   {
      for(ktrack2=1;ktrack2<= ktrack1;ktrack2++)
      {
         if(tdumvec[ktrack2-1] > tdumvec[ktrack2])
         {
            hold=tdumvec[ktrack2];
            tdumvec[ktrack2]=tdumvec[ktrack2-1];
            tdumvec[ktrack2-1]=hold;
         }
      }
   }
#ifdef DEBUG
   printf("Sorted: Low to high\n");
   for(ktrack1=0;ktrack1<=veclength;ktrack1++)
   {
      printf("%3d %8.5f\n",ktrack1, tdumvec[ktrack1]);
   }
#endif
}
       
/* prepare a vector to be sorted */
void prepare(tdumvec,thispar,treatpar)
double *tdumvec;
int thispar, treatpar;
{
   int veclength,ktrack1,ktrack2;
   
   veclength= (((nseqs-1)*nseqs)/2) -1; 
   for(ktrack1=0;ktrack1<=veclength;ktrack1++)
   {
      if(treatpar==0)
      {
         tdumvec[ktrack1]= data[thispar][ktrack1];
      }            
      else
      {
         if(treatpar>0)
         {
            tdumvec[ktrack1]= 
               data[abs(treatpar)][ktrack1]/data[thispar][ktrack1];
         }
         else
         {
            tdumvec[ktrack1]= 
               data[thispar][ktrack1]/data[abs(treatpar)][ktrack1];
         } 
      }
   }
}
