/* This program reads an upper triangular pairwise distance matrix
   from the file "indata".  

   It assumes the matrix format is to follow a pairwise distance by
   the standard error of that distance.  Then, this program creates
   NUMSIMS resampled distance matrices.  Let, d(i,j) and e(i,j)
   respectively be the pairwise distance and standard error
   between taxa i and j for the original data set.  A resampled distance
   matrix entry for taxa i and j will be denoted d'(i,j) and e'(i,j).
   The value d'(i,j) is sampled from a normal distribution with mean
   d(i,j) and standard deviation e(i,j).  The value e'(i,j) is
   equal to e(i,j).  The context and further details of this
   resampling procedure are found in Thorne and Kishino (MBE,9:1148-1162).  
*/

#include <math.h>
#include <stdio.h>

#define MULT 1664525 /* a constant needed by the random number generator */ 
#define MAXSPECIES 24 /* Most taxa that can be in the distance matrix */
#define NUMSIMS 100  /* Number of resampled data sets to be created */
#define NAMELEN 10 /* Number of characters per taxon name */ 

long seed; /* random number seed */
double dis[MAXSPECIES][MAXSPECIES]; /* distance between 2 taxa */
double dev[MAXSPECIES][MAXSPECIES]; /* standard deviation of distance */
char name[MAXSPECIES][NAMELEN]; /* taxa names */
int numspecies; /* actual number of species in data set */

main()
{
   void prtdata(); /* print the original data */
   void simdata(); /* simulate and print a resampled data set */
   void readdata(); /* read the original data */
   double rnd(); /* generate a random seed */

   int i;
   FILE *inseed; /* contains the seed */

   readdata(); 
   inseed=fopen("inseed","r");
   fscanf(inseed,"%ld",&seed);
   fclose(inseed);
   printf("seed= %11ld numspecies= %2d NUMSIMS=%4d\n",seed,numspecies,
   NUMSIMS);
   prtdata();
   for(i=1;i<=100;i++) rnd();
   for(i=0;i<NUMSIMS;i++)
   {
      simdata();
   }
   
   inseed=fopen("inseed","w");
   fprintf(inseed,"%ld",seed); /* put in a new random seed */
   fclose(inseed);
}

void prtdata()
{
   int ktrack1,ktrack2;

   printf("\n");
   
   for(ktrack1=0;ktrack1<numspecies;ktrack1++)
   {
      for(ktrack2=0;ktrack2<NAMELEN;ktrack2++)
      {
	 printf("%1c",name[ktrack1][ktrack2]);
      }
      for(ktrack2=ktrack1+1;ktrack2< numspecies;ktrack2++)
      {
	 printf(" %8.4f %7.4f", dis[ktrack1][ktrack2], dev[ktrack1][ktrack2]);
      }
      printf("\n");
   }
}

void simdata()
{
   double normal();
   FILE *sampdata;
   static int flag=0;
   int ktrack1,ktrack2;

   if (flag==0)
   {
      /* if this is the first simulated data set, then start a new file */
      sampdata=fopen("sampdata","w"); 
      flag=1;
   }
   else
   {
      /* append file "sampdata" if this is not first simulated data set */
      sampdata=fopen("sampdata","a");
   }
   fprintf(sampdata,"%d\n",numspecies);
   for(ktrack1=0;ktrack1<numspecies;ktrack1++)
   {
      for(ktrack2=0;ktrack2<NAMELEN;ktrack2++)
      {
	 fprintf(sampdata,"%1c",name[ktrack1][ktrack2]);
      }
      for(ktrack2=ktrack1+1;ktrack2< numspecies;ktrack2++)
      {
	 fprintf(sampdata," %8.4f %7.4f", normal(dis[ktrack1][ktrack2],
	 dev[ktrack1][ktrack2]), dev[ktrack1][ktrack2]);
      }
      fprintf(sampdata,"\n");
   }
   fclose(sampdata);
}

void readdata()
{
   FILE *indata;
   int ktrack1,ktrack2;
   char tempchar;

   indata=fopen("indata","r");
   fscanf(indata,"%d",&numspecies);
   for(ktrack1=0;ktrack1<numspecies;ktrack1++)
   {
      for(ktrack2=0;ktrack2< NAMELEN;ktrack2++)
      {
	 tempchar=getc(indata);
	 if (tempchar=='\n') 
            tempchar=getc(indata);
	 name[ktrack1][ktrack2]=tempchar;
      }
      for(ktrack2=ktrack1+1;ktrack2< numspecies;ktrack2++)
      {
	 fscanf(indata,"%lf%lf", &dis[ktrack1][ktrack2],
	 &dev[ktrack1][ktrack2]);
      }
   }
   fclose(indata);
}

#define PI 3.1415927

/* return a normal random number with mean "inmu" and standard 
   deviation "insigma" */
double normal(inmu,insigma)
double inmu,insigma;
{
   double rnd();

   static int onemore=0;
   static double rankeep;
   double u1,u2,x1,x2;

   if (onemore==0)
   {
      u1=rnd();
      u2=rnd();
      x1= sqrt(-2.0*log(u1))*cos(2.0*PI*u2);
      x2= sqrt(-2.0*log(u1))*sin(2.0*PI*u2);
      rankeep=x2;
   }
   else
   {
      x1=rankeep;
   }
   onemore= 1-onemore;
   return(inmu+insigma*x1);
}

/* this procedure requires a constant:
	#define MULT 1664525
			and a long variable:
        long seed;
			to work */
double rnd()
{
double rndom;
seed = seed * MULT;
rndom = seed / 4294967296.0 + 0.5;
return(rndom);
}

